#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_TSL2561_U.h>

/* LDR fotoellenállás kalibrálása TLS2561 szenzorral
 Az alábbi program az Adafruit TLS2561 szenzorához
 készített mintaprogramjának kibővített változata.   
 
 A TLS2561 modul bekötése:
 =========================
 Connect SCL to analog 5
 Connect SDA to analog 4
 Connect VDD to 3.3V DC (vagy +5V ha van 3,3V-os stabilizátor rajta!)
 Connect GROUND to common ground
 
 Az LDR bekötése:
 --------------------  
 GND -/\/\/----+--(LDR)-- +3.3V
 10k     |
 to A0 
 
 I2C Address
 ===========
 Ha a TLS2561 ADDR lába lebeg, akkor 0x39 az I2C címe
 Ha az ADDR láb magas szintre van húzva, akkor 0x49 az I2C cím
 Ha az ADDR láb alacsony szintre van húzva, akkor 0x29 az I2C cím
 
 Adafruit TLS2561 programkönyvtár és mintapélda:
 ================================================ 
 Link: https://github.com/adafruit/Adafruit_TSL2561
 Szükséges hozzá az Adafruit_Sensor library is:
 Link: https://github.com/adafruit/Adafruit_Sensor
 */

Adafruit_TSL2561_Unified tsl = Adafruit_TSL2561_Unified(TSL2561_ADDR_LOW, 12345);

/**************************************************************************/
/*
    Displays some basic information on this sensor from the unified
 sensor API sensor_t type (see Adafruit_Sensor for more information)
 */
/**************************************************************************/
void displaySensorDetails(void)
{
  sensor_t sensor;
  tsl.getSensor(&sensor);
  Serial.println("------------------------------------");
  Serial.print  ("Sensor:       "); 
  Serial.println(sensor.name);
  Serial.print  ("Driver Ver:   "); 
  Serial.println(sensor.version);
  Serial.print  ("Unique ID:    "); 
  Serial.println(sensor.sensor_id);
  Serial.print  ("Max Value:    "); 
  Serial.print(sensor.max_value); 
  Serial.println(" lux");
  Serial.print  ("Min Value:    "); 
  Serial.print(sensor.min_value); 
  Serial.println(" lux");
  Serial.print  ("Resolution:   "); 
  Serial.print(sensor.resolution); 
  Serial.println(" lux");  
  Serial.println("------------------------------------");
  Serial.println("");
  delay(500);
}

/**************************************************************************/
/*
    Configures the gain and integration time for the TSL2561
 */
/**************************************************************************/
void configureSensor(void)
{
  /* You can also manually set the gain or enable auto-gain support */
  // tsl.setGain(TSL2561_GAIN_1X);      /* No gain ... use in bright light to avoid sensor saturation */
  // tsl.setGain(TSL2561_GAIN_16X);     /* 16x gain ... use in low light to boost sensitivity */
  tsl.enableAutoRange(true);            /* Auto-gain ... switches automatically between 1x and 16x */

  /* Changing the integration time gives you better sensor resolution (402ms = 16-bit data) */
  tsl.setIntegrationTime(TSL2561_INTEGRATIONTIME_13MS);      /* fast but low resolution */
  // tsl.setIntegrationTime(TSL2561_INTEGRATIONTIME_101MS);  /* medium resolution and speed   */
  // tsl.setIntegrationTime(TSL2561_INTEGRATIONTIME_402MS);  /* 16-bit data but slowest conversions */

  /* Update these values depending on what you've set above! */
  Serial.println("------------------------------------");
  Serial.print  ("Gain:         "); 
  Serial.println("Auto");
  Serial.print  ("Timing:       "); 
  Serial.println("13 ms");
  Serial.println("------------------------------------");
}

/**************************************************************************/
/*
    Arduino setup function (automatically called at startup)
 */
/**************************************************************************/
int sensorValue = 0;  // variable to store the value coming from the analog sensor 

void setup(void) 
{
  Serial.begin(9600);
  Serial.println("Light Sensor Test"); 
  Serial.println("");

  /* Initialise the sensor */
  if(!tsl.begin())
  {
    /* There was a problem detecting the ADXL345 ... check your connections */
    Serial.print("Ooops, no TSL2561 detected ... Check your wiring or I2C ADDR!");
    while(1);
  }

  /* Display some basic information on this sensor */
  displaySensorDetails();

  /* Setup the sensor gain and integration time */
  configureSensor();

  /* We're ready to go! */
  Serial.println("");
}

/**************************************************************************/
/*
    Arduino loop function, called once 'setup' is complete (your own code
 should go here)
 */
/**************************************************************************/
void loop(void) 
{  
  /* Get a new sensor event */
  sensors_event_t event;
  tsl.getEvent(&event);

  /* Display the results (light is measured in lux) */
  if (event.light)
  {
    Serial.print(event.light); 
    Serial.println(" lux");
  }
  else
  {
    /* If event.light = 0 lux the sensor is probably saturated
     and no reliable data could be generated! */
    Serial.println("Sensor overload");
  }
  //--- LDR kalibrálás betoldása ------------------------------
  int sensorValue = analogRead(A0);  //ADC beolvasása
  Serial.print("A0 = ");
  Serial.print(sensorValue);
  Serial.print(" V0: ");
  float voltage = sensorValue*4.66/1023; //Vref ténylegesen = 4.66 V
  Serial.print(voltage,3);
  float Rx = 10000.0;
  if(sensorValue>0) {
    Rx = 33.0/voltage-10;     //Osztó táplálása 3.3 V-ról történik!
  }  
  Serial.print(" V  Resistance = ");
  Serial.print(Rx,1);
  Serial.println(" kOhm");
  delay(2000);        // 2 mp várakozás
}

